import java.io.*;
import java.util.Hashtable;
import java.util.StringTokenizer ;

public class myClassLoader extends ClassLoader {
    private Hashtable classes = new Hashtable();
    protected String path ;
    protected String extension ;

    public myClassLoader( String path, String extension ) {
      this.path = path ;
      this.extension = extension ;
    }

    /* Carica il contenuto di un file .class */
    private byte getClassBytes(String className)[] {
        System.out.println("Lettura dati da file per la classe "+className);
        byte result[];
        try {
            result = loadClassBytes( className ) ;
            return result;
        } catch (Exception e) {
            return null;
        }
    }

    protected byte[] loadClassBytes( String className )
      throws IOException {
        String fileSeparator = System.getProperty( "file.separator" ) ;
        className = className.replace( '.', fileSeparator.charAt(0) ) +
          "." + extension ;
        System.out.println( "Nome file : " + className ) ;
        String classpath = System.getProperty( "java.class.path" ) ;
        StringTokenizer st = new StringTokenizer( 
          classpath, System.getProperty("path.separator") ) ;
        File classFile = null ;
        File dir = null ;
        while( st.hasMoreTokens() ) {
          dir = new File( st.nextToken() + fileSeparator + path ) ;
          System.out.println( "directory : " + dir ) ;
          classFile = new File( dir, className ) ;
          if ( classFile.exists() )
            break ;
        }       
        
        FileInputStream fi = new FileInputStream( classFile ) ;
        byte result[] = new byte[ fi.available() ] ;
        fi.read( result ) ;
        return result ;
    }

    /* Questa e' la versione chiamata */
    public Class loadClass(String className) throws ClassNotFoundException {
        return (loadClass(className, true));
    }

    /* Questa e' la versione completa */
    public synchronized Class loadClass(String className, boolean resolveIt)
        throws ClassNotFoundException {
        Class result;
        byte  classData[];

        System.out.println("Caricamento della classe : "+className);

        /* Prima si controlla la cache */
        result = (Class)classes.get(className);
        if (result != null) {
            System.out.println("Utilizzata la copia nella cache.");
            return result;
        }

        /* Prima si prova col primordial class loader */
        try {
            result = super.findSystemClass(className);
            System.out.println("Classe di sistema (CLASSPATH)");
            return result;
        } catch (ClassNotFoundException e) {
            System.out.println("Non e' una classe di sistema.");
            if ( className.startsWith( "java.") ) {
              System.out.println("Classe pericolosa : " + className ) ;
              System.out.println( "Caricamento abortito" ) ;
              throw new ClassNotFoundException();
            }
        }

        /* poi si prova a caricare la classe dal path specificato */
        classData = getClassBytes(className);
        if (classData == null) {
            throw new ClassNotFoundException();
        }

        /* viene eseguito il parsing, e costruito l'oggetto class  */
        result = defineClass(className,classData, 0, classData.length);
        if (result == null) {
            throw new ClassFormatError();
        }

        if (resolveIt) {
            resolveClass(result);
        }

        /* Si aggiorna la cache */
        classes.put(className, result);
        System.out.println("Classe caricata : " +className);
        return result;
    }
}
